//===-- FrontendActions.h - Useful Frontend Actions -------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

#ifndef LLVM_CLANG_REWRITE_FRONTENDACTIONS_H
#define LLVM_CLANG_REWRITE_FRONTENDACTIONS_H

#include "clang/Frontend/FrontendAction.h"

#include "clang/Frontend/CompilerInstance.h"
#include <string>
#include <vector>

#include <sys/stat.h>

namespace clang {
class FixItRewriter;
class FixItOptions;

//===----------------------------------------------------------------------===//
// AST Consumer Actions
//===----------------------------------------------------------------------===//

class HTMLPrintAction : public ASTFrontendAction {
protected:
  virtual ASTConsumer *CreateASTConsumer(CompilerInstance &CI,
                                         llvm::StringRef InFile);
};

class FixItAction : public ASTFrontendAction {
protected:
  llvm::OwningPtr<FixItRewriter> Rewriter;
  llvm::OwningPtr<FixItOptions> FixItOpts;

  virtual ASTConsumer *CreateASTConsumer(CompilerInstance &CI,
                                         llvm::StringRef InFile);

  virtual bool BeginSourceFileAction(CompilerInstance &CI,
                                     llvm::StringRef Filename);

  virtual void EndSourceFileAction();

  virtual bool hasASTFileSupport() const { return false; }

public:
  FixItAction();
  ~FixItAction();
};

class RewriteObjCAction : public ASTFrontendAction {
protected:
  virtual ASTConsumer *CreateASTConsumer(CompilerInstance &CI,
                                         llvm::StringRef InFile);
};

class RewriteMacrosAction : public PreprocessorFrontendAction {
protected:
  void ExecuteAction();
};

class RewriteTestAction : public PreprocessorFrontendAction {
protected:
  void ExecuteAction();
};

class RewriteScopedRefptrAction : public ASTFrontendAction {
protected:
  virtual ASTConsumer *CreateASTConsumer(CompilerInstance &CI,
                                         llvm::StringRef InFile);

#if 0
// doh, this is called before the rewriter finishes. so this isn't just hacky, it's wrong too
  virtual void EndSourceFileAction() {
//fprintf(stderr, "done with %s, %s\n", getCurrentFile().c_str(), getCompilerInstance().getFrontendOpts().OutputFile.c_str());
//HACK
    if (getCompilerInstance().getDiagnostics().getNumErrors() == 0) {

struct stat s;

stat(getCompilerInstance().getFrontendOpts().OutputFile.c_str(), &s);

if (!s.st_size) return;

      rename(getCompilerInstance().getFrontendOpts().OutputFile.c_str(),
             getCurrentFile().c_str());
    } else {
fprintf(stderr, "had errors; not copying file.\n");
    }
  }
#endif
};

}  // end namespace clang

#endif
